#include "sphere.hh"

void Sphere::TryHit(HitRec &rec, const XYZ &eye, const XYZ &dir) const
{
    const XYZ V = eye - center;
    const FLOAT DV = dir.DotProduct(V);
    const FLOAT D2 = dir.squared();
    const FLOAT SQ = DV*DV - D2 * (V.squared() - radius*radius);
    
    rec.washit = false;
    
    if(SQ < EPSILON)
        return;
    
    const FLOAT SQt = SQRT(SQ);
    
    //fprintf(stderr, "Hit!\n");
    
    const FLOAT depth1 = (-DV + SQt) / D2;
    const FLOAT depth2 = (-DV - SQt) / D2;
    
#if 1
    const FLOAT sel = depth1 < depth2 ? depth1 : depth2;
    if(sel < EPSILON)return;
    rec.point = eye + dir * sel;
#else
    if(depth1 < EPSILON)
    {
        if(depth2 < EPSILON)return;
        rec.point = eye + dir * depth2;
    }
    else if(depth2 < EPSILON)
        rec.point = eye + dir * depth1;
    else
        rec.point = eye + (dir * (depth1 < depth2 ? depth1 : depth2));
#endif
    rec.normal = (rec.point - center) / radius;
    rec.Pigm = Pigm;
    rec.washit = true;
    
    rec.ior = 1.54; // Quartz
    
#if 0
    /* Texturizing the surface */
    FLOAT phi = 0.5 + ASIN(rec.normal.y) / M_PI;
    FLOAT len = rec.normal.x * rec.normal.x + rec.normal.z * rec.normal.z;
    FLOAT theta;
    
    if(len <= EPSILON)
        theta = 0.0; /* poles, any x coord is ok */
    else if(rec.normal.z == 0.0)
           theta = (rec.normal.x > 0.0) ? 0.0 : 1.0;
    else
    {
        theta = ACOS(rec.normal.x / SQRT(len));
        if(rec.normal.z < 0.0)theta = (M_PI * 2) - theta;
        theta /= (M_PI * 2);
    }
    
    if((fmodl(phi,   0.2) < 0.05)
     ^ (fmodl(theta, 0.2) < 0.05))
    {
        rec.Pigm.Set(1,0,0);
    }
#endif
}

const XYZ Sphere::Min() const
{
    return XYZ(center.x - radius, center.y - radius, center.z - radius);
}

const XYZ Sphere::Max() const
{
    return XYZ(center.x + radius, center.y + radius, center.z + radius);
}
